from dataclasses import dataclass
from typing import Optional
from datetime import datetime


@dataclass
class Evaluation:
    id: str
    title: str
    internal_name: Optional[str]
    description: Optional[str]
    status: str
    created_time: datetime
    updated_time: datetime

    @staticmethod
    def from_dict(data: dict) -> "Evaluation":
        required_keys = ["id", "title", "status", "created_time", "updated_time"]
        for key in required_keys:
            if key not in data:
                raise ValueError(f"Invalid data format for Evaluation: {data}")

        return Evaluation(
            id=data["id"],
            title=data["title"],
            internal_name=data["internal_name"],
            description=data["description"],
            status=data["status"],
            created_time=datetime.fromisoformat(data["created_time"].replace("Z", "+00:00")),
            updated_time=datetime.fromisoformat(data["updated_time"].replace("Z", "+00:00")),
        )

    def to_dict(self) -> dict:
        created_time_str = self.created_time.strftime("%Y-%m-%dT%H:%M:%S.%f")[:-3] + "Z"
        updated_time_str = self.updated_time.strftime("%Y-%m-%dT%H:%M:%S.%f")[:-3] + "Z"
        return {
            "id": self.id,
            "title": self.title,
            "internal_name": self.internal_name,
            "description": self.description,
            "status": self.status,
            "created_time": created_time_str,
            "updated_time": updated_time_str,
        }



import os
import podonos
import unittest
from unittest import mock

from podonos.core.file import File
from podonos.common.constant import *


# Mocks HTTP GET request.
def mocked_requests_get(*args, **kwargs):
    class MockResponse:
        def __init__(self, response_text, response_json, status_code):
            self.text = response_text
            self.json_response = response_json
            self.status_code = status_code

        def json(self):
            return self.json_response

    if "/customers/verify/api-key" in args[0]:
        # API key verification
        return MockResponse("true", None, 200)

    if "/version/sdk" in args[0]:
        # SDK versions
        version_response = dict(latest="0.1.5", recommended="0.1.4", minimum="0.1.0")
        return MockResponse(None, version_response, 200)

    if "/customers/uploading-presigned-url" in args[0]:
        return MockResponse("https://fake.podonos.com/my_url1", None, 200)

    return MockResponse(None, None, 404)


def mocked_requests_post(*args, **kwargs):
    class MockResponse:
        def __init__(self, response_text, response_json, status_code):
            self.text = response_text
            self.json_response = response_json
            self.status_code = status_code

        def json(self):
            return self.json_response

        def raise_for_status(self):
            return

    if "/evaluations" in args[0]:
        # evaluations
        evaluation_response = dict(
            id="mock_id",
            title="mock_title",
            internal_name="mock_internal_name",
            description="mock_desc",
            status="mock_status",
            created_time="2024-05-21T06:18:09.659270Z",
            updated_time="2024-05-22T06:18:09.659270Z",
        )
        return MockResponse(None, evaluation_response, 200)

    return MockResponse(None, None, 404)


class TestPodonos(unittest.TestCase):

    @mock.patch("requests.get", side_effect=mocked_requests_get)
    def test_init(self, mock_get):
        # Invalid api_key
        invalid_api_key = "1"
        with self.assertRaises(Exception) as context:
            _ = podonos.init(invalid_api_key)

        # Valid api_key
        valid_api_key = "12345678"
        valid_client = podonos.init(valid_api_key)
        self.assertTrue(isinstance(valid_client, podonos.Client))

    @mock.patch("requests.get", side_effect=mocked_requests_get)
    def test_init_api_key_env(self, mock_get):
        # Valid api_key_env
        os.environ[PODONOS_API_KEY] = "ABCDEFG"
        valid_client = podonos.init()
        self.assertTrue(isinstance(valid_client, podonos.Client))

    @mock.patch("requests.get", side_effect=mocked_requests_get)
    def test_init_both_api_keys(self, mock_get):
        # Valid api_key_env
        os.environ[PODONOS_API_KEY] = "ABCDEFG"
        valid_api_key = "12345678"
        valid_client = podonos.init(valid_api_key)
        self.assertTrue(isinstance(valid_client, podonos.Client))


class TestPodonosClient(unittest.TestCase):

    @mock.patch("requests.get", side_effect=mocked_requests_get)
    def setUp(self, mock_get) -> None:
        valid_api_key = "1234567890"
        self._mock_client = podonos.init(api_key=valid_api_key)

        self.config = {"name": "my_new_model_03272024_p1_k2_en_us", "desc": "Updated model", "type": "NMOS", "lan": "en-us"}

    @mock.patch("requests.get", side_effect=mocked_requests_get)
    @mock.patch("requests.post", side_effect=mocked_requests_post)
    def test_create_evaluator(self, mock_get, mock_post):
        # Missing name is allowed.
        etor = self._mock_client.create_evaluator(desc=self.config["desc"], type=self.config["type"], lan=self.config["lan"])
        self.assertIsNotNone(etor)

        # Too short name.
        with self.assertRaises(ValueError) as context:
            _ = self._mock_client.create_evaluator(name="a", desc=self.config["desc"], type=self.config["type"], lan=self.config["lan"])

        # Missing description is allowed.
        etor = self._mock_client.create_evaluator(name=self.config["name"], type=self.config["type"], lan=self.config["lan"])
        self.assertIsNotNone(etor)

        # Unknown type.
        with self.assertRaises(ValueError) as context:
            _ = self._mock_client.create_evaluator(name=self.config["name"], desc=self.config["desc"], type="unknown_type", lan=self.config["lan"])

        # Missing language is allowed.
        etor = self._mock_client.create_evaluator(name=self.config["name"], desc=self.config["desc"], type=self.config["type"])
        self.assertIsNotNone(etor)

        # Invalid language.
        with self.assertRaises(ValueError) as context:
            _ = self._mock_client.create_evaluator(name=self.config["name"], desc=self.config["desc"], type=self.config["type"], lan="abcd")

        # Valid configuration
        etor = self._mock_client.create_evaluator(
            name=self.config["name"], desc=self.config["desc"], type=self.config["type"], lan=self.config["lan"]
        )
        self.assertIsNotNone(etor)

        # P.808
        etor = self._mock_client.create_evaluator(name=self.config["name"], desc=self.config["desc"], type="P808", lan=self.config["lan"])
        self.assertIsNotNone(etor)

        # SMOS
        etor = self._mock_client.create_evaluator(name=self.config["name"], desc=self.config["desc"], type="SMOS", lan=self.config["lan"])
        self.assertIsNotNone(etor)


class TestPodonosEvaluator(unittest.TestCase):

    @unittest.skip
    @mock.patch("requests.get", side_effect=mocked_requests_get)
    def test_evaluator(self, mock_get):
        valid_api_key = "1234567890"
        self._mock_client = podonos.init(api_key=valid_api_key)

        self.config = {"name": "my_new_model_03272024_p1_k2_en_us", "desc": "Updated model", "type": "NMOS", "lan": "en-us"}

        etor = self._mock_client.create_evaluator(name="new_model", desc="", type="NMOS", lan="en-au")
        self.assertIsNotNone(etor)

        some_file = os.path.join(os.path.dirname(__file__), "../speech_0_0.mp3")
        etor.add_file(File(path=some_file, model_tag='my_new_model', tags=["unknown_file,new_model"]))
        etor.close()


if __name__ == "__main__":
    unittest.main(verbosity=2)



from dataclasses import dataclass
from typing import List, Union, Literal


@dataclass
class StimulusStatsFile:
    name: str
    model_tag: str
    tags: List[str]
    type: Union[Literal["A"], Literal["B"]]

    @staticmethod
    def from_dict(data: dict) -> "StimulusStatsFile":
        required_keys = ["name", "model_tag", "tags", "type"]
        for key in required_keys:
            if key not in data:
                raise ValueError(f"Invalid data format for StimulusStatsFile: {data}")
        return StimulusStatsFile(name=data["name"], model_tag=data["model_tag"], tags=data["tags"], type=data["type"])

    def to_dict(self) -> dict:
        return {"name": self.name, "model_tag": self.model_tag, "tags": self.tags, "type": self.type}


@dataclass
class StimulusStats:
    files: List[StimulusStatsFile]
    mean: float
    median: float
    std: float
    ci_90: float
    ci_95: float
    ci_99: float

    @staticmethod
    def from_dict(data: dict) -> "StimulusStats":
        required_keys = ["files", "mean", "median", "std", "ci_90", "ci_95", "ci_99"]
        for key in required_keys:
            if key not in data:
                raise ValueError(f"Invalid data format for StimulusStats: {data}")

        files = [StimulusStatsFile.from_dict(file) for file in data["files"]]
        return StimulusStats(
            files=files,
            mean=data["mean"],
            median=data["median"],
            std=data["std"],
            ci_90=data["ci_90"],
            ci_95=data["ci_95"],
            ci_99=data["ci_99"],
        )

    def to_dict(self) -> dict:
        return {
            "files": [file.to_dict() for file in self.files],
            "mean": self.mean,
            "median": self.median,
            "std": self.std,
            "ci_90": self.ci_90,
            "ci_95": self.ci_95,
            "ci_99": self.ci_99,
        }



from typing import Union, List

from podonos.common.enum import EvalType, QuestionFileType
from podonos.core.api import APIClient
from podonos.core.base import *
from podonos.core.config import EvalConfig
from podonos.core.evaluator import Evaluator
from podonos.core.file import File
from podonos.errors.error import NotSupportedError


class SingleStimulusEvaluator(Evaluator):
    def __init__(
        self,
        supported_evaluation_types: List[EvalType],
        api_client: APIClient,
        eval_config: Union[EvalConfig, None] = None,
    ):
        log.check(api_client, "api_client is not initialized")
        super().__init__(api_client, eval_config)
        self._supported_evaluation_types = supported_evaluation_types

    def add_file(self, file: File) -> None:
        """Add new file for speech evaluation.
        The file may be either in {wav, mp3} format. The file will be securely uploaded to
        Podonos service system.

        Args:
            file: File object including the path, the model tag, the other tags, and the script.

        Example:
        If you want to evaluate each audio file separately (e.g., Naturalness MOS):
            add_file(file=File(path='./test.wav', model_tag='my_new_model1', tags=['male', 'generated'],
                               script='hello there'))

        Returns: None

        Raises:
            ValueError: if this function is called before calling init()
            FileNotFoundError: if a given file is not found.
        """
        log.check(file, "file is not set")

        if not self._initialized:
            raise ValueError("Try to add file once the evaluator is closed.")

        eval_config = self._get_eval_config()
        if eval_config.eval_type in self._supported_evaluation_types:
            if eval_config.eval_use_annotation and file.script is None:
                raise ValueError(
                    "Annotation evaluation is enabled (eval_use_annotation=True), "
                    "but no script is provided in File. Please provide a corresponding script."
                )

            audio = self._set_audio(
                file=file,
                group=None,
                type=QuestionFileType.STIMULUS,
                order_in_group=0,
            )
            self._eval_audios.append([audio])
            self._upload_one_file(
                evaluation_id=self.get_evaluation_id(),
                remote_object_name=audio.remote_object_name,
                path=audio.path,
            )

    def add_files(self, file0: File, file1: File) -> None:
        raise NotSupportedError("The 'add_files' is only supported for group evaluations like "
                                "{'CMOS', 'DMOS', 'PREF'}")



from typing import Any, Dict, Optional, List

from requests import HTTPError

from podonos.common.enum import EvalType
from podonos.core.api import APIClient
from podonos.core.base import *
from podonos.core.config import EvalConfig, EvalConfigDefault
from podonos.core.evaluation import Evaluation
from podonos.core.evaluator import Evaluator
from podonos.core.stimulus_stats import StimulusStats
from podonos.evaluators.double_stimuli_evaluator import DoubleStimuliEvaluator
from podonos.evaluators.single_stimulus_evaluator import SingleStimulusEvaluator


class Client:
    """Podonos Client class. Used for creating individual evaluator and managing the evaluations."""

    _api_client: APIClient
    _initialized: bool = False

    def __init__(self, api_client: APIClient):
        self._api_client = api_client
        self._initialized = True

    def create_evaluator(
        self,
        name: Optional[str] = None,
        desc: Optional[str] = None,
        type: str = EvalConfigDefault.TYPE.value,
        lan: str = EvalConfigDefault.LAN.value,
        granularity: float = EvalConfigDefault.GRANULARITY,
        num_eval: int = EvalConfigDefault.NUM_EVAL,
        due_hours: int = EvalConfigDefault.DUE_HOURS,
        use_annotation: bool = EvalConfigDefault.USE_ANNOTATION,
        auto_start: bool = EvalConfigDefault.AUTO_START,
        max_upload_workers: int = EvalConfigDefault.MAX_UPLOAD_WORKERS,
    ) -> Evaluator:
        """Creates a new evaluator with a unique evaluation session ID.
        For the language code, see https://docs.dyspatch.io/localization/supported_languages/

        Args:
            name: This session name. Its length must be > 1. If empty, a random name is used. Optional.
            desc: Description of this session. Optional.
            type: Evaluation type. Default: NMOS
            lan: Human language for this audio. One of those in Language. Default: en-us
            granularity: Granularity of the evaluation scales. Either {1, 0.5}
            num_eval: The minimum number of repetition for each audio evaluation. Should be >=1. Default: 10.
            due_hours: An expected number of days of finishing this mission and getting the evaluation report.
                        Must be >= 12. Default: 12.
            use_annotation: Enable detailed annotation on script for detailed rating reasoning.
            auto_start: The evaluation start automatically if True. Otherwise, manually start in the workspace.
            max_upload_workers: The maximum number of upload workers. Must be a positive integer. Default: 20

        Returns:
            Evaluator instance.

        Raises:
            ValueError: if this function is called before calling init().
        """

        if not self._initialized:
            raise ValueError("This function is called before initialization.")

        if not EvalType.is_eval_type(type):
            raise ValueError("Not supported evaluation types. Use one of the "
                             "{'NMOS', 'QMOS', 'P808', 'SMOS', 'PREF'}")

        eval_config = EvalConfig(
            name=name,
            desc=desc,
            type=type,
            lan=lan,
            granularity=granularity,
            num_eval=num_eval,
            due_hours=due_hours,
            use_annotation=use_annotation,
            auto_start=auto_start,
            max_upload_workers=max_upload_workers,
        )
        evaluator = None
        if type in [EvalType.SMOS.value, EvalType.PREF.value]:
            evaluator = DoubleStimuliEvaluator(
                supported_evaluation_types=[EvalType.SMOS, EvalType.PREF],
                api_client=self._api_client,
                eval_config=eval_config,
            )
        else:
            evaluator = SingleStimulusEvaluator(
                supported_evaluation_types=[EvalType.NMOS, EvalType.QMOS, EvalType.P808],
                api_client=self._api_client,
                eval_config=eval_config,
            )
        log.check(isinstance(evaluator, Evaluator))
        return evaluator

    def get_evaluation_list(self) -> List[Dict[str, Any]]:
        """Gets a list of evaluations.

        Args: None

        Returns:
            Evaluation containing all the evaluation info
        """
        log.check(self._api_client)
        try:
            response = self._api_client.get("evaluations")
            response.raise_for_status()
            evaluations = [Evaluation.from_dict(evaluation) for evaluation in response.json()]
            return [evaluation.to_dict() for evaluation in evaluations]
        except Exception as e:
            raise HTTPError(f"Failed to get evaluation list: {e}")

    def get_stats_dict_by_id(self, evaluation_id: str) -> List[Dict[str, Any]]:
        """Gets a list of evaluation statistics referenced by id.

        Args:
            evaluation_id: Evaluation id. See get_evaluation_list() above.

        Returns:
            List of statistics for the evaluation.
        """
        log.check(self._api_client)
        try:
            response = self._api_client.get(f"evaluations/{evaluation_id}/stats")
            if response.status_code == 400:
                log.info(f"Bad Request: The {evaluation_id} is an invalid evaluation id")
                return []

            response.raise_for_status()
            stats = [StimulusStats.from_dict(stats) for stats in response.json()]
            return [stat.to_dict() for stat in stats]
        except Exception as e:
            raise HTTPError(f"Failed to get evaluation stats: {e}")

    def download_stats_csv_by_id(self, evaluation_id: str, output_path: str) -> None:
        """Downloads the evaluation statistics into CSV referenced by id.

        Args:
            evaluation_id: Evaluation id. See get_evaluation_list() above.
            output_path: Path to the output CSV.

        Returns: None
        """
        log.check_ne(evaluation_id, "")
        log.check_ne(output_path, "")
        stats = self.get_stats_dict_by_id(evaluation_id)
        with open(output_path, "w") as f:
            f.write("name,tags,type,mean,median,std,ci_90,ci_95,ci_99\n")
            for stat in stats:
                for file in stat["files"]:
                    tags = ";".join(file["tags"])
                    f.write(
                        f"{file['name']},{tags},{file['type']},{stat['mean']},{stat['median']},{stat['std']},"
                        f"{stat['ci_90']},{stat['ci_95']},{stat['ci_99']}\n"
                    )



import unittest
from podonos.core.config import EvalConfig


class TestEvalConfig(unittest.TestCase):
    def setUp(self):
        self.eval_config = EvalConfig()

    def test_set_eval_use_annotation_valid_types(self):
        valid_eval_types = ["NMOS", "QMOS", "P808"]

        for eval_type in valid_eval_types:
            result = self.eval_config._validate_eval_use_annotation(eval_use_annotation=True, eval_type=eval_type)
            self.assertTrue(result)

    def test_set_eval_use_annotation_invalid_type(self):
        invalid_eval_type = "SMOS"

        with self.assertRaises(ValueError) as context:
            self.eval_config._validate_eval_use_annotation(eval_use_annotation=True, eval_type=invalid_eval_type)

        self.assertEqual(str(context.exception), '"eval_type" must be one of {NMOS, QMOS, P808} when using "use_annotation"')



import os
import unittest
import requests

from datetime import datetime
from typing import Optional
from unittest.mock import Mock, patch, MagicMock

from podonos.common.enum import QuestionFileType
from podonos.common.exception import HTTPError
from podonos.core.api import APIClient
from podonos.core.config import EvalConfig
from podonos.core.evaluation import Evaluation
from podonos.core.evaluator import Evaluator
from podonos.core.file import File
from tests.test_audio import TESTDATA_SPEECH_CH1_MP3


class MockEvaluator(Evaluator):
    def __init__(self, api_client=Mock(spec=APIClient), eval_config: Optional[EvalConfig] = None):
        super().__init__(api_client, eval_config)

    def add_file(self, file: File) -> None:
        pass

    def add_files(self, target: File, ref: File) -> None:
        pass

    def _create_evaluation(self) -> Evaluation:
        evaluation_config = {
            "id": "mock_id",
            "title": "mock_title",
            "internal_name": "mock_internal_name",
            "description": "mock_desc",
            "status": "mock_status",
            "created_time": "2024-05-21T06:18:09.659270Z",
            "updated_time": "2024-05-22T06:18:09.659270Z",
        }
        evaluation = Evaluation.from_dict(evaluation_config)
        return evaluation


class TestEvaluator(unittest.TestCase):

    def setUp(self):
        self.eval_config = EvalConfig(type="NMOS")
        self.evaluator = MockEvaluator(eval_config=self.eval_config)

    def test_get_eval_config_success(self):
        eval_config = EvalConfig(type="NMOS")
        evaluator = MockEvaluator(eval_config=eval_config)

        result = evaluator._get_eval_config()
        self.assertEqual(result, eval_config)

    def test_get_eval_config_not_initialized(self):
        evaluator = MockEvaluator(eval_config=None)

        with self.assertRaises(ValueError) as context:
            evaluator._get_eval_config()

        self.assertEqual(str(context.exception), "Evaluator is not initialized")

    def test_create_evaluation_success(self):
        mock_response = MagicMock()
        mock_response.raise_for_status.side_effect = None  # No exception
        mock_response.json.return_value = {
            "id": "123",
            "title": "title",
            "internal_name": None,
            "description": None,
            "status": "DRAFT",
            "created_time": datetime.now().isoformat(),
            "updated_time": datetime.now().isoformat(),
        }

        eval_config = EvalConfig("NMOS")
        evaluator = MockEvaluator(api_client=MagicMock(), eval_config=eval_config)
        evaluator._api_client.post.return_value = mock_response  # type: ignore

        evaluation = evaluator._create_evaluation()

        self.assertEqual(evaluation.id, "mock_id")

    @patch("os.path.isfile", return_value=True)
    @patch("os.access", return_value=True)
    def test_set_audio(self, mock_access, mock_isfile):
        audio_file = File(path=TESTDATA_SPEECH_CH1_MP3, model_tag="model1", tags=["tag1", "tag2"], script="Hello World")
        group = "group1"
        type = QuestionFileType.STIMULUS
        order_in_group = 0
        audio = self.evaluator._set_audio(audio_file, group, type, order_in_group)
        posix_style_path = TESTDATA_SPEECH_CH1_MP3.replace("\\", "/")

        self.assertEqual(audio.path, audio_file.path)
        self.assertEqual(audio.name, posix_style_path)
        self.assertEqual(audio.model_tag, audio_file.model_tag)
        self.assertEqual(audio.tags, audio_file.tags)
        self.assertEqual(audio.script, audio_file.script)
        self.assertEqual(audio.group, group)
        self.assertEqual(audio.type, type)
        self.assertEqual(audio.order_in_group, order_in_group)

    @patch("os.path.isfile")
    @patch("os.access")
    def test_validate_path_success(self, mock_access, mock_isfile):
        mock_isfile.return_value = True
        mock_access.return_value = True

        evaluator = MockEvaluator(eval_config=EvalConfig(type="NMOS"))

        valid_path = "/valid/path/to/file.txt"
        result = evaluator._validate_path(valid_path)
        self.assertEqual(result, valid_path)

        mock_isfile.assert_called_once_with(valid_path)
        mock_access.assert_called_once_with(valid_path, os.R_OK)

    @patch("os.path.isfile")
    @patch("os.access")
    def test_validate_path_file_not_exist(self, mock_access, mock_isfile):
        mock_isfile.return_value = False
        evaluator = MockEvaluator(api_client=MagicMock(), eval_config=EvalConfig(type="NMOS"))

        invalid_path = "/invalid/path/to/file.txt"
        with self.assertRaises(FileNotFoundError) as context:
            evaluator._validate_path(invalid_path)

        self.assertIn("doesn't exist", str(context.exception))

        mock_isfile.assert_called_once_with(invalid_path)
        mock_access.assert_not_called()

    @patch("os.path.isfile")
    @patch("os.access")
    def test_validate_path_not_readable(self, mock_access, mock_isfile):
        mock_isfile.return_value = True
        mock_access.return_value = False

        evaluator = MockEvaluator(eval_config=EvalConfig(type="NMOS"))

        unreadable_path = "/unreadable/path/to/file.txt"
        with self.assertRaises(FileNotFoundError) as context:
            evaluator._validate_path(unreadable_path)

        self.assertIn("isn't readable", str(context.exception))

        mock_isfile.assert_called_once_with(unreadable_path)
        mock_access.assert_called_once_with(unreadable_path, os.R_OK)

    def test_paths(self):
        test_cases = [
            # Test case: paths with backslashes
            {
                "original_path": r"C:\Users\Example\Documents\file.txt",
                "remote_object_path": r"C:\Files\Other\file.txt",
                "expected_original_path": "C:/Users/Example/Documents/file.txt",
                "expected_remote_object_path": "C:/Files/Other/file.txt",
            },
            # Test case: paths with forward slashes
            {
                "original_path": "C:/Users/Example/Documents/file.txt",
                "remote_object_path": "C:/Files/Other/file.txt",
                "expected_original_path": "C:/Users/Example/Documents/file.txt",
                "expected_remote_object_path": "C:/Files/Other/file.txt",
            },
            # Test case: paths with mixed slashes
            {
                "original_path": r"C:\Users/Example\Documents/file.txt",
                "remote_object_path": r"C:/Files\Other\file.txt",
                "expected_original_path": "C:/Users/Example/Documents/file.txt",
                "expected_remote_object_path": "C:/Files/Other/file.txt",
            },
        ]

        for case in test_cases:
            posix_original_path, posix_remote_object_path = self.evaluator._process_original_path_and_remote_object_path_into_posix_style(
                case["original_path"], case["remote_object_path"]
            )

            self.assertEqual(posix_original_path, case["expected_original_path"])
            self.assertEqual(posix_remote_object_path, case["expected_remote_object_path"])


if __name__ == "__main__":
    unittest.main()



import pytest
from unittest.mock import Mock

from typing import List, Optional

from podonos.common.enum import EvalType
from podonos.core.api import APIClient
from podonos.core.config import EvalConfig
from podonos.core.evaluation import Evaluation
from podonos.core.file import File
from podonos.errors.error import NotSupportedError
from podonos.evaluators.double_stimuli_evaluator import DoubleStimuliEvaluator
from tests.test_audio import TESTDATA_SPEECH_TWO_CH1_WAV, TESTDATA_SPEECH_TWO_CH2_WAV


class MockDoubleStimuliEvaluator(DoubleStimuliEvaluator):
    def __init__(self, supported_evaluation_types: List[EvalType], api_client=Mock(spec=APIClient), eval_config: Optional[EvalConfig] = None):
        super().__init__(supported_evaluation_types, api_client, eval_config)

    def _create_evaluation(self) -> Evaluation:
        evaluation_config = {
            "id": "mock_id",
            "title": "mock_title",
            "internal_name": "mock_internal_name",
            "description": "mock_desc",
            "status": "mock_status",
            "created_time": "2024-05-21T06:18:09.659270Z",
            "updated_time": "2024-05-22T06:18:09.659270Z",
        }
        evaluation = Evaluation.from_dict(evaluation_config)
        return evaluation


class TestDoubleStimuliEvaluator:
    def setup_method(self):
        self.api_client = Mock(spec=APIClient)
        self.eval_config = EvalConfig(type="SMOS")
        self.evaluator = MockDoubleStimuliEvaluator(
            supported_evaluation_types=[EvalType.SMOS], api_client=self.api_client, eval_config=self.eval_config
        )

    def test_add_files_without_init(self):
        self.evaluator._initialized = False
        target_audio_config = File(path="generated.wav", model_tag="model_target", tags=["target"], script="This is the target audio file")
        ref_audio_config = File(path="original.wav", model_tag="model_ref", tags=["reference"], script=None, is_ref=True)

        with pytest.raises(ValueError) as excinfo:
            self.evaluator.add_files(file0=target_audio_config, file1=ref_audio_config)

        assert "evaluator is not initialized" in str(excinfo.value)

    def test_add_file_not_supported(self):
        with pytest.raises(NotSupportedError) as excinfo:
            self.evaluator.add_file(File(path="1.wav", model_tag="model1", tags=["test"]))
        assert "only supported" in str(excinfo.value)

    def test_add_files_unsupported_eval_type(self):
        self.evaluator._initialized = True
        self.eval_config._eval_type = EvalType.CMOS

        target = File(path="target.wav", model_tag="model_target", tags=["generated"])
        ref = File(path="ref.wav", model_tag="model_ref", tags=["human"], is_ref=True)

        with pytest.raises(ValueError) as excinfo:
            self.evaluator.add_files(file0=target, file1=ref)

        assert "Unsupported" in str(excinfo.value)

    def test_add_files_missing_ref(self):
        self.evaluator._initialized = True
        self.eval_config._eval_type = EvalType.CMOS

        file0 = File(path="file0.wav", model_tag="model0", tags=["file0"], script="We are the world")
        file1 = File(path="file1.wav", model_tag="model1", tags=["file1"])

        with pytest.raises(ValueError) as excinfo:
            self.evaluator.add_files(file0=file0, file1=file1)

        assert "Unsupported" in str(excinfo.value)

    def test_generate_random_group_name(self):
        group_name = self.evaluator._generate_random_group_name()
        parts = group_name.split("_")
        assert len(parts) == 2
        assert parts[0].isdigit()
        assert len(parts[1]) == 36



import unittest

from datetime import datetime
from unittest.mock import MagicMock

from podonos.core.upload_manager import UploadManager
from tests.test_audio import TESTDATA_SPEECH_CH1_MP3


class TestUploadManager(unittest.TestCase):

    def test_upload_manager(self):
        mock_response = MagicMock()
        mock_response.raise_for_status.side_effect = None  # No exception
        mock_response.json.return_value = {
            "id": "123",
            "title": "title",
            "internal_name": None,
            "description": None,
            "status": "DRAFT",
            "created_time": datetime.now().isoformat(),
            "updated_time": datetime.now().isoformat(),
        }

        upload_manager = UploadManager(api_client=MagicMock(), max_workers=1)
        upload_manager._api_client.post.return_value = mock_response  # type: ignore

        evaluation_id = "AAAA1234"
        remote_object_name = "ABCD1234"
        path = TESTDATA_SPEECH_CH1_MP3
        upload_manager.add_file_to_queue(evaluation_id, remote_object_name, path)

        self.assertTrue(upload_manager.wait_and_close())


if __name__ == "__main__":
    unittest.main()



from typing import Optional, Any

from podonos.core.base import *


# Class to represent a question with a title and an optional description
class Question:
    _title: str
    _description: Optional[str]

    def __init__(self, title: str, description: Optional[str]):
        log.check_notnone(title)
        log.check_ne(title, "")
        self._title = title
        self._description = description

    @property
    def title(self) -> str:
        return self._title

    @property
    def description(self) -> Optional[str]:
        return self._description

    def to_dict(self) -> "dict[str, Any]":  # type: ignore
        return {"title": self._title, "description": self._description}


# Define a Query class that encapsulates a Question object and a Option object in the future
class Query:
    _question: Question

    def __init__(self, question: Question):
        self._question = question

    @property
    def question(self) -> Question:
        return self._question

    @property
    def title(self) -> str:
        return self._question.title

    @property
    def description(self) -> Optional[str]:
        return self._question.description

    def to_dict(self) -> "dict[str, Any]":  # type: ignore
        return {"question": self._question.to_dict()}



import time
import uuid
from typing import Union, List

from podonos.common.enum import EvalType, QuestionFileType
from podonos.core.api import APIClient
from podonos.core.base import *
from podonos.core.config import EvalConfig
from podonos.core.evaluator import Evaluator
from podonos.core.file import File
from podonos.errors.error import NotSupportedError


class DoubleStimuliEvaluator(Evaluator):
    def __init__(
        self,
        supported_evaluation_types: List[EvalType],
        api_client: APIClient,
        eval_config: Union[EvalConfig, None] = None,
    ):
        log.check(api_client, "api_client is not initialized")
        super().__init__(api_client, eval_config)
        self._supported_evaluation_types = supported_evaluation_types

    def add_file(self, file: File) -> None:
        raise NotSupportedError("The 'add_file' is only supported in single file evaluation types: " "{'NMOS', 'QMOS', 'P808'}")

    def add_files(self, file0: File, file1: File) -> None:
        """Adds files for speech evaluation in an ordered or unordered way. If the evaluation requires an order of
        the input files, the order is kept strictly. The files will be securely uploaded to Podonos service system.

        Args:
            file0: The first audio file configuration.
            file1: The second audio file configuration.

        Example:
        If you want to evaluate audio files together (e.g., Comparative MOS):
            f0 = File(path="/path/to/generated.wav", model_tag='my_new_model1', tags=['male', 'english'], is_ref=True)
            f1 = File(path="/path/to/original.wav", model_tag='my_new_model2', tags=['male', 'english', 'param1'])
            add_files(file0=f0, file1=f1)

        Returns: None

        Raises:
            ValueError: if this function is called before calling init().
            FileNotFoundError: if a given file is not found.
        """
        log.check(file0, "file0 is not set")
        log.check(file1, "file1 is not set")

        if not self._initialized:
            raise ValueError("Try to add_files once the evaluator is not initialized.")

        eval_config = self._get_eval_config()
        if eval_config.eval_type not in self._supported_evaluation_types:
            raise ValueError(f"Unsupported evaluation type: {eval_config.eval_type}")
        if eval_config.eval_type not in [EvalType.SMOS, EvalType.PREF, EvalType.CMOS, EvalType.DMOS]:
            raise ValueError("The add_files is used for such evaluations that require multiple files.")

        group = self._generate_random_group_name()
        if eval_config.eval_type == EvalType.PREF:
            # Files are ordered stimulus.
            audio0 = self._set_audio(file=file0, group=group, type=QuestionFileType.STIMULUS, order_in_group=0)
            audio1 = self._set_audio(file=file1, group=group, type=QuestionFileType.STIMULUS, order_in_group=1)
        elif eval_config.eval_type == EvalType.SMOS:
            # Files are unordered stimulus.
            audio0 = self._set_audio(file=file0, group=group, type=QuestionFileType.STIMULUS, order_in_group=0)
            audio1 = self._set_audio(file=file1, group=group, type=QuestionFileType.STIMULUS, order_in_group=1)
        else:  # CMOS, DMOS
            # Files are ordered stimulus, one of them must be a reference.
            if file0.is_ref == file1.is_ref:
                raise ValueError("One of the input files must be set as a reference.")
            audio0_type = QuestionFileType.REF if file0.is_ref else QuestionFileType.STIMULUS
            audio1_type = QuestionFileType.REF if file1.is_ref else QuestionFileType.STIMULUS
            audio0 = self._set_audio(file=file0, group=group, type=audio0_type, order_in_group=0)
            audio1 = self._set_audio(file=file1, group=group, type=audio1_type, order_in_group=1)
        self._eval_audios.append([audio0, audio1])

        self._upload_one_file(evaluation_id=self.get_evaluation_id(), remote_object_name=audio0.remote_object_name, path=audio0.path)
        self._upload_one_file(evaluation_id=self.get_evaluation_id(), remote_object_name=audio1.remote_object_name, path=audio1.path)

    @staticmethod
    def _generate_random_group_name() -> str:
        current_time_milliseconds = int(time.time() * 1000)
        random_uuid = uuid.uuid4()
        name = f"{current_time_milliseconds}_{random_uuid}"
        log.check_gt(len(name), 5)
        log.check_lt(len(name), 100)
        return name



"""
Default enum values across whole SDK
"""

from enum import Enum


class EvalType(Enum):
    NMOS = "NMOS"
    QMOS = "QMOS"
    P808 = "P808"
    SMOS = "SMOS"
    PREF = "PREF"
    CMOS = "CMOS"
    DMOS = "DMOS"

    def get_type(self) -> str:
        if self.value == "PREF":
            return "SPEECH_PREFERENCE"
        return f"SPEECH_{self.value}"

    @staticmethod
    def is_eval_type(eval_type: str) -> bool:
        return any([item for item in EvalType if item.value == eval_type])


class Language(Enum):
    ENGLISH_AMERICAN = "en-us"
    ENGLISH_BRITISH = "en-gb"
    ENGLISH_AUSTRALIAN = "en-au"
    ENGLISH_CANADIAN = "en-ca"
    KOREAN = "ko-kr"
    MANDARIN = "zh-cn"
    SPANISH_SPAIN = "es-es"
    SPANISH_MEXICO = "es-mx"
    FRENCH = "fr-fr"
    GERMAN = "de-de"
    JAPANESE = "ja-jp"
    ITALIAN = "it-it"
    POLISH = "pl-pl"
    AUDIO = "audio"


class QuestionFileType(str, Enum):
    STIMULUS = "STIMULUS"
    REF = "REF"
    META = "META"



import pytest
from unittest.mock import Mock

from typing import List, Optional

from podonos.common.enum import EvalType
from podonos.core.api import APIClient
from podonos.core.config import EvalConfig
from podonos.core.evaluation import Evaluation
from podonos.core.file import File
from podonos.errors.error import NotSupportedError
from podonos.evaluators.single_stimulus_evaluator import SingleStimulusEvaluator


class MockDoubleStimuliEvaluator(SingleStimulusEvaluator):
    def __init__(self, supported_evaluation_types: List[EvalType], api_client=Mock(spec=APIClient), eval_config: Optional[EvalConfig] = None):
        super().__init__(supported_evaluation_types, api_client, eval_config)

    def _create_evaluation(self) -> Evaluation:
        evaluation_config = {
            "id": "mock_id",
            "title": "mock_title",
            "internal_name": "mock_internal_name",
            "description": "mock_desc",
            "status": "mock_status",
            "created_time": "2024-05-21T06:18:09.659270Z",
            "updated_time": "2024-05-22T06:18:09.659270Z",
        }
        evaluation = Evaluation.from_dict(evaluation_config)
        return evaluation


class TestSingleStimulusEvaluator:
    def setup_method(self):
        self.api_client = Mock(spec=APIClient)
        self.eval_config = EvalConfig(type="NMOS")
        self.evaluator = MockDoubleStimuliEvaluator(
            supported_evaluation_types=[EvalType.NMOS, EvalType.QMOS, EvalType.P808], api_client=self.api_client, eval_config=self.eval_config
        )

    def test_add_file_before_initialized(self):
        self.evaluator._initialized = False
        with pytest.raises(ValueError) as excinfo:
            self.evaluator.add_file(file=File(path="./male.wav", model_tag="model1", tags=["male"]))
        assert "Try to add file once the evaluator is closed." in str(excinfo.value)

    def test_add_files_not_supported(self):
        target_audio_config = File(path="target.wav", model_tag="model2", tags=["target"])
        ref_audio_config = File(path="ref.wav", model_tag="model1", tags=["ref"])
        with pytest.raises(NotSupportedError) as excinfo:
            self.evaluator.add_files(file0=target_audio_config, file1=ref_audio_config)
        assert "only supported" in str(excinfo.value)



import unittest
from podonos.core.query import Query, Question


class TestQuery(unittest.TestCase):
    def test_query_with_not_null(self):
        TITLE = "This is custom question"
        DESCRIPTION = "This is custom description"
        QUESTION_DICT = {"title": TITLE, "description": DESCRIPTION}
        QUERY_DICT = {"question": QUESTION_DICT}

        question = Question(TITLE, DESCRIPTION)
        query = Query(question=question)
        self.assertEqual(query.title, TITLE)
        self.assertEqual(query.description, DESCRIPTION)
        self.assertEqual(question.title, TITLE)
        self.assertEqual(question.description, DESCRIPTION)
        self.assertEqual(query.question, question)
        self.assertEqual(query.question.to_dict(), QUESTION_DICT)
        self.assertEqual(query.to_dict(), QUERY_DICT)

    def test_query_with_null(self):
        TITLE = "This is custom question"
        DESCRIPTION = None
        QUESTION_DICT = {"title": TITLE, "description": DESCRIPTION}
        QUERY_DICT = {"question": QUESTION_DICT}

        question = Question(TITLE, DESCRIPTION)
        query = Query(question=question)
        self.assertEqual(query.title, TITLE)
        self.assertEqual(query.description, DESCRIPTION)
        self.assertEqual(question.title, TITLE)
        self.assertEqual(question.description, DESCRIPTION)
        self.assertEqual(query.question, question)
        self.assertEqual(query.question.to_dict(), QUESTION_DICT)
        self.assertEqual(query.to_dict(), QUERY_DICT)


if __name__ == "__main__":
    unittest.main(verbosity=2)



"""
Default constant values across whole SDK
"""


# Text colors on terminal
class TerminalColor:
    BOLD = ""
    ENDC = ""
    FAIL = ""
    OK = ""
    WARN = ""


# Podonos homepage
PODONOS_HOME = "https://www.podonos.com/"

# Podonos docs API key
PODONOS_DOCS_API_KEY = "https://www.podonos.com/docs/apikey"

# API key environment variable
PODONOS_API_KEY = "PODONOS_API_KEY"

# Podonos Workspace
PODONOS_WORKSPACE = "https://workspace.podonos.com"

# Podonos API base URL
PODONOS_API_BASE_URL = "https://prod.podonosapi.com"

# Contact email
PODONOS_CONTACT_EMAIL = "hello@podonos.com"



class NotSupportedError(Exception):
    """Exception raised for unsupported operations."""

    def __init__(self, message="This operation is not supported"):
        self.message = message
        super().__init__(self.message)


class InvalidFileError(Exception):
    """Exception raised for invalid file input."""

    def __init__(self, message="This file is invalid"):
        self.message = message
        super().__init__(self.message)



import os
import podonos
from podonos.core.client import Client, Evaluator, SingleStimulusEvaluator, DoubleStimuliEvaluator

import unittest
from unittest import mock
from unittest.mock import patch


def mocked_requests_post(*args, **kwargs):
    class MockResponse:
        def __init__(self, response_text, response_json, status_code):
            self.text = response_text
            self.json_response = response_json
            self.status_code = status_code

        def json(self):
            return self.json_response

        def raise_for_status(self):
            return

    if "/evaluations" in args[0]:
        # Evaluation list
        evaluation_list = dict(
                id="mock_id",
                title="mock_title",
                internal_name="mock_internal_name",
                description="mock_desc",
                status="mock_status",
                created_time="2024-05-21T06:18:09.659270Z",
                updated_time="2024-05-21T06:18:09.659270Z",
            )
        return MockResponse(None, evaluation_list, 200)

    return MockResponse(None, None, 404)


# Mocks HTTP GET request.
def mocked_requests_get(*args, **kwargs):
    class MockResponse:
        def __init__(self, response_text, response_json, status_code):
            self.text = response_text
            self.json_response = response_json
            self.status_code = status_code

        def json(self):
            return self.json_response

        def raise_for_status(self):
            return

    if "/customers/verify/api-key" in args[0]:
        # API key verification
        return MockResponse("true", None, 200)

    if "/version/sdk" in args[0]:
        # SDK versions
        version_response = dict(latest="0.1.5", recommended="0.1.4", minimum="0.1.0")
        return MockResponse(None, version_response, 200)

    if "/evaluations" in args[0] and "/stats" in args[0]:
        # Stats by id
        evaluation_stats = [
            dict(files=[{"name": "tr16.wav", "model_tag": "my_model", "tags": ["generated"], "type": "A"}],
                 mean=3.4, median=3.5, std=1.07, ci_90=1.14, ci_95=1.48, ci_99=1.53)
        ]
        return MockResponse(None, evaluation_stats, 200)

    if "/evaluations" in args[0]:
        # Evaluation list
        evaluation_list = [
            dict(
                id="mock_id",
                title="mock_title",
                internal_name="mock_internal_name",
                description="mock_desc",
                status="mock_status",
                created_time="2024-05-21T06:18:09.659270Z",
                updated_time="2024-05-21T06:18:09.659270Z",
            )
        ]
        return MockResponse(None, evaluation_list, 200)

    return MockResponse(None, None, 404)


class TestEvaluationClient(unittest.TestCase):

    @mock.patch("requests.get", side_effect=mocked_requests_get)
    @mock.patch("requests.post", side_effect=mocked_requests_post)
    def test_single_stimulus_evaluator_creation(self, mock_get, mock_post):
        valid_api_key = "1234567890"
        self._mock_client = podonos.init(api_key=valid_api_key)
        name = "good_test"
        desc = "detailed description"
        type = "NMOS"
        lan = "it-it"
        granularity = 1.0
        num_eval = 8
        due_hours = 12
        auto_start = False
        max_upload_workers = 10
        etor = self._mock_client.create_evaluator(
            name=name, desc=desc, type=type, lan=lan, granularity=granularity, num_eval=num_eval,
            due_hours=due_hours, auto_start=auto_start, max_upload_workers=max_upload_workers)
        self.assertTrue(isinstance(etor, SingleStimulusEvaluator))

    @mock.patch("requests.get", side_effect=mocked_requests_get)
    @mock.patch("requests.post", side_effect=mocked_requests_post)
    def test_double_stimuli_evaluator_creation(self, mock_get, mock_post):
        valid_api_key = "1234567890"
        self._mock_client = podonos.init(api_key=valid_api_key)
        name = "good_test"
        desc = "detailed description"
        type = "PREF"
        lan = "ko-kr"
        granularity = 1.0
        num_eval = 5
        due_hours = 12
        auto_start = False
        max_upload_workers = 10
        etor = self._mock_client.create_evaluator(
            name=name, desc=desc, type=type, lan=lan, granularity=granularity, num_eval=num_eval,
            due_hours=due_hours, auto_start=auto_start, max_upload_workers=max_upload_workers)
        self.assertTrue(isinstance(etor, DoubleStimuliEvaluator))

    @mock.patch("requests.get", side_effect=mocked_requests_get)
    def test_evaluation_list(self, mock_get):
        valid_api_key = "1234567890"
        self._mock_client = podonos.init(api_key=valid_api_key)
        response = self._mock_client.get_evaluation_list()
        self.assertTrue(isinstance(response, list))
        self.assertTrue(len(response) > 0)
        json = response[0]
        self.assertTrue("id" in json)
        self.assertTrue("title" in json)
        self.assertTrue("internal_name" in json)
        self.assertTrue("description" in json)
        self.assertTrue("status" in json)
        self.assertTrue("created_time" in json)
        self.assertTrue("updated_time" in json)

    @mock.patch("requests.get", side_effect=mocked_requests_get)
    def test_stimulus_stats_by_id(self, mock_get):
        valid_api_key = "1234567890"
        self._mock_client = podonos.init(api_key=valid_api_key)
        response = self._mock_client.get_stats_dict_by_id(evaluation_id="mock_id")
        self.assertTrue(isinstance(response, list))
        self.assertTrue(len(response) > 0)
        json = response[0]
        self.assertTrue("files" in json)
        self.assertTrue("mean" in json)
        self.assertTrue("median" in json)
        self.assertTrue("std" in json)
        self.assertTrue("ci_90" in json)
        self.assertTrue("ci_95" in json)
        self.assertTrue("ci_99" in json)


class TestEvaluationClientApiKey(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        api_key_envs = {"PODONOS_API_KEY": "ABCD123ENV"}
        cls.env = patch.dict(in_dict=os.environ, values=api_key_envs, clear=True)
        cls.env.start()

    @classmethod
    def tearDownClass(cls):
        super().tearDownClass()
        cls.env.stop()

    @mock.patch("requests.get", side_effect=mocked_requests_get)
    def test_env_api_key_only(self, mock_get):
        api_key_env = os.getenv("PODONOS_API_KEY")
        self.assertEqual(api_key_env, "ABCD123ENV")
        mock_client = podonos.init()
        self.assertTrue(isinstance(mock_client, Client))

    @mock.patch("requests.get", side_effect=mocked_requests_get)
    def test_both_keys(self, mock_get):
        api_key_env = os.getenv("PODONOS_API_KEY")
        self.assertEqual(api_key_env, "ABCD123ENV")
        mock_client = podonos.init(api_key="ABCD123PARAM")
        self.assertTrue(isinstance(mock_client, Client))


if __name__ == "__main__":
    unittest.main(verbosity=2)



import re
import unittest
from unittest.mock import patch, MagicMock
from podonos.core.api import APIClient, APIVersion


class TestAPIVersion(unittest.TestCase):
    def teatVersions(self):
        minimum = "0.1.0"
        recommended = "0.1.5"
        latest = "0.2.0"
        ver = APIVersion(minimum, recommended, latest)
        self.assertEqual(minimum, ver.minimum)
        self.assertEqual(recommended, ver.recommended)
        self.assertEqual(latest, ver.latest)


class TestAPIClient(unittest.TestCase):
    def setUp(self):
        self.api_key = "test_api_key"
        self.api_url = "http://testapi.com"
        self.client = APIClient(self.api_key, self.api_url)

    def test_params(self):
        self.assertEqual(self.api_key, self.client.api_key)
        self.assertEqual(self.api_url, self.client.api_url)

    @patch("requests.get")
    def test_get_success(self, mock_get):
        mock_response = MagicMock()
        mock_response.text = "true"
        mock_get.return_value = mock_response

        response = self.client.get("test-endpoint")
        self.assertEqual(response.text, "true")
        mock_get.assert_called_once_with(f"{self.api_url}/test-endpoint", headers=self.client._headers, params=None)

    @patch("requests.post")
    def test_post_success(self, mock_post):
        mock_response = MagicMock()
        mock_response.status_code = 200
        mock_post.return_value = mock_response

        data = {"key": "value"}
        response = self.client.post("test-endpoint", data)
        self.assertEqual(response.status_code, 200)
        mock_post.assert_called_once_with(f"{self.api_url}/test-endpoint", headers=self.client._headers, json=data)

    @patch("requests.put")
    def test_put_success(self, mock_put):
        mock_response = MagicMock()
        mock_response.status_code = 200
        mock_put.return_value = mock_response

        data = {"key": "value"}
        response = self.client.put("test-endpoint", data)
        self.assertEqual(response.status_code, 200)
        mock_put.assert_called_once_with(f"{self.api_url}/test-endpoint", headers=self.client._headers, json=data)

    @patch("podonos.core.api.APIClient._check_minimum_version")
    @patch("podonos.core.api.APIClient.get")
    def test_initialize_success(self, mock_get, mock_check_minimum_version):
        mock_response = MagicMock()
        mock_response.text = "true"
        mock_get.return_value = mock_response
        mock_check_minimum_version.return_value = True

        result = self.client.initialize()
        self.assertTrue(result)
        mock_get.assert_called_once_with(f"customers/verify/api-key")

    @patch("podonos.core.api.APIClient._check_minimum_version")
    @patch("podonos.core.api.APIClient.get")
    def test_initialize_invalid_api_key(self, mock_get, mock_check_minimum_version):
        mock_response = MagicMock()
        mock_response.text = "false"
        mock_get.return_value = mock_response
        mock_check_minimum_version.return_value = True

        with self.assertRaises(ValueError) as context:
            self.client.initialize()

        self.assertIn("Invalid API key", str(context.exception))
        mock_get.assert_called_once_with("customers/verify/api-key")

    def test_add_headers(self):
        self.client.add_headers("New-Header", "HeaderValue")
        self.assertIn("New-Header", self.client._headers)
        self.assertEqual(self.client._headers["New-Header"], "HeaderValue")

    @patch("importlib.metadata.version")
    def test_get_podonos_version_success(self, mock_version):
        mock_version.return_value = "1.2.3"

        result = self.client._get_podonos_version()
        self.assertEqual(result, "1.2.3")
        mock_version.assert_called_once_with("podonos")

    def test_get_content_type(self):
        path_wav = '/a/b/123.wav'
        self.assertEqual("audio/wav", self.client._get_content_type_by_filename(path_wav))

        path_wav = '/a/b/456.mp3'
        self.assertEqual("audio/mpeg", self.client._get_content_type_by_filename(path_wav))

        path_wav = '/a/b/789.json'
        self.assertEqual("application/json", self.client._get_content_type_by_filename(path_wav))

        path_wav = '/a/b/abc.obj'
        self.assertEqual("application/octet-stream", self.client._get_content_type_by_filename(path_wav))

    def test_package_version(self):
        version_str = self.client._get_podonos_version()
        # Test the version string is in the format of "<decimal_version>.<decimal_subversion>"
        self.assertTrue(re.search(r'\d+\.\d+', version_str))


if __name__ == "__main__":
    unittest.main()



import os
import soundfile as sf
from pathlib import Path
from typing import Tuple, Optional, Dict, Any, List

from podonos.common.enum import QuestionFileType
from podonos.core.base import *
from podonos.errors.error import InvalidFileError


class AudioMeta:
    _nchannels: int
    _framerate: int
    _duration_in_ms: int

    def __init__(self, path: str) -> None:
        log.check_notnone(path)
        self._nchannels, self._framerate, self._duration_in_ms = self._set_audio_meta(path)
        log.check_gt(self._nchannels, 0)
        log.check_gt(self._framerate, 0)
        log.check_gt(self._duration_in_ms, 0)

    @property
    def nchannels(self) -> int:
        return self._nchannels

    @property
    def framerate(self) -> int:
        return self._framerate

    @property
    def duration_in_ms(self) -> int:
        return self._duration_in_ms

    def _set_audio_meta(self, path: str) -> Tuple[int, int, int]:
        """Gets info from an audio file.

        Returns:
            nchannels: Number of channels
            framerate: Number of frames per second. Same as the sampling rate.
            duration_in_ms: Total length of the audio in milliseconds

        Raises:
            FileNotFoundError: if the file is not found.
            wave.Error: if the file doesn't read properly.
            AssertionError: if the file format is not wav.
        """
        log.check_notnone(path)
        log.check_ne(path, "")
        log.check(os.path.isfile(path), f"{path} doesn't exist")
        log.check(os.access(path, os.R_OK), f"{path} isn't readable")

        # Check if this is wav or mp3.
        suffix = Path(path).suffix
        support_file_type = [".wav", ".mp3", ".flac"]
        assert suffix in support_file_type, f"Unsupported file format: {path}. It must be wav or mp3."
        if suffix in support_file_type:
            return self._get_audio_info(path)
        return 0, 0, 0

    def _get_audio_info(self, filepath: str) -> Tuple[int, int, int]:
        """Gets info from a wave file.

        Returns:
            nchannels: Number of channels
            framerate: Number of frames per second. Same as the sampling rate.
            duration_in_ms: Total length of the audio in milliseconds

        Raises:
            FileNotFoundError: if the file is not found.
            wave.Error: if the file doesn't read properly.
        """
        log.check_notnone(filepath)
        log.check_ne(filepath, "")

        f = sf.SoundFile(filepath)
        nframes = f.frames
        nchannels = f.channels
        framerate = f.samplerate
        log.check_gt(nframes, 0)
        log.check_gt(nchannels, 0)
        log.check_gt(framerate, 0)

        duration_in_ms = int(nframes * 1000.0 / float(framerate))
        log.check_gt(duration_in_ms, 0)
        return nchannels, framerate, duration_in_ms


class Audio:
    _path: str
    _name: str
    _remote_object_name: str
    _metadata: AudioMeta
    _script: Optional[str] = None
    _is_ref: bool = False
    _upload_start_at: Optional[str] = None
    _upload_finish_at: Optional[str] = None
    _model_tag: str
    _tags: Optional[List[str]] = None
    _group: Optional[str] = None
    _type: QuestionFileType = QuestionFileType.STIMULUS
    _order_in_group: int = 0

    def __init__(
        self,
        path: str,
        name: str,
        remote_object_name: str,
        script: Optional[str],
        is_ref: bool,
        model_tag: str,
        tags: Optional[List[str]],
        group: Optional[str],
        type: QuestionFileType,
        order_in_group: int,
    ) -> None:
        log.check_notnone(path)
        log.check_ne(path, "")
        log.check(os.path.isfile(path), f"{path} doesn't exist")
        log.check(os.access(path, os.R_OK), f"{path} isn't readable")
        log.check_notnone(model_tag)
        log.check(model_tag, "")
        log.check_notnone(name)
        log.check_notnone(remote_object_name)
        log.check_ge(order_in_group, 0)

        self._path = path
        self._name = name
        self._remote_object_name = remote_object_name
        self._script = script
        self._metadata = AudioMeta(path)
        self._model_tag = model_tag
        self._is_ref = is_ref
        self._tags = tags
        self._group = group
        self._type = type
        self._order_in_group = order_in_group

    @property
    def path(self) -> str:
        return self._path

    @property
    def name(self) -> str:
        return self._name

    @property
    def remote_object_name(self) -> str:
        return self._remote_object_name

    @property
    def script(self) -> Optional[str]:
        return self._script

    @property
    def model_tag(self) -> str:
        return self._model_tag

    @property
    def tags(self) -> Optional[List[str]]:
        return self._tags

    @property
    def is_ref(self) -> bool:
        return self._is_ref

    @property
    def group(self) -> Optional[str]:
        return self._group

    @property
    def metadata(self) -> AudioMeta:
        return self._metadata

    @property
    def type(self) -> QuestionFileType:
        return self._type

    @property
    def order_in_group(self) -> int:
        return self._order_in_group

    def set_upload_at(self, start_at: str, finish_at: str) -> None:
        log.check_notnone(start_at)
        log.check_notnone(finish_at)
        log.check_ne(start_at, "")
        log.check_ne(finish_at, "")

        self._upload_start_at = start_at
        self._upload_finish_at = finish_at

    def to_dict(self) -> Dict[str, Any]:
        return {
            "name": self._name,
            "remote_name": self._remote_object_name,
            "nchannels": self._metadata.nchannels,
            "framerate": self._metadata.framerate,
            "duration_in_ms": self._metadata.duration_in_ms,
            "upload_start_at": self._upload_start_at,
            "upload_finish_at": self._upload_finish_at,
            "model_tag": self._model_tag,
            "is_ref": self._is_ref,
            "tag": self._tags,
            "type": self._type,
            "script": self._script,
            "group": self._group,
            "order_in_group": self._order_in_group,
        }

    def to_create_file_dict(self) -> Dict[str, Any]:
        return {
            "original_uri": self._path,
            "processed_uri": self._remote_object_name,
            "duration": self._metadata.duration_in_ms,
            "model_tag": self._model_tag,
            "is_ref": self._is_ref,
            "tags": self._tags,
            "type": self._type,
            "script": self._script,
            "group": self._group,
            "order_in_group": self._order_in_group,
        }



import atexit
import datetime
import requests
import queue
import threading
import time

from concurrent.futures import ThreadPoolExecutor
from threading import Event
from tqdm import tqdm
from typing import Optional

from podonos.core.api import APIClient
from podonos.common.exception import HTTPError
from podonos.core.base import *


class UploadManager:
    """Concurrent file upload manager.
    Internally creates multiple threads, and manages the uploading status.
    """

    # File path queue
    # TODO: use a file queue.
    _queue: Optional[queue.Queue] = None
    # Total number of files added to the uploading queue.
    _total_files: int = 0
    _total_uploaded: int = 0

    _pbar: Optional[tqdm] = None
    # Event to all the uploader threads
    _worker_event: Optional[Event] = None
    # Master daemon thread. Alive until the manager closes.
    _daemon_thread: Optional[threading.Thread] = None
    # API client for
    _api_client: Optional[APIClient] = None
    # Manager status. True if the manager is ready.
    _status: bool = False
    # Maximum number of uploader worker threads
    _max_workers: int = 1
    #
    _upload_start: Optional[dict] = None
    _upload_finish: Optional[dict] = None

    def get_upload_time(self):
        if not self._upload_start or not self._upload_finish:
            raise ValueError("Upload Fail")

        return self._upload_start, self._upload_finish

    def __init__(self, api_client: APIClient, max_workers: int) -> None:
        log.check(api_client, "api_client is not initialized")

        self._upload_start = dict()
        self._upload_finish = dict()
        self._api_client = api_client
        self._queue = queue.Queue()
        self._total_files = 0
        self._max_workers = max_workers
        self._worker_event = Event()
        self._daemon_thread = threading.Thread(target=self._uploader_daemon, daemon=True)
        self._daemon_thread.start()
        self._status = True

        atexit.register(self.wait_and_close)

    def _uploader_daemon(self) -> None:
        log.debug(f"Uploader daemon is running with {self._max_workers} workers")
        with ThreadPoolExecutor(max_workers=self._max_workers) as executor:
            for index in range(self._max_workers):
                future = executor.submit(self._upload_worker, index, self._worker_event)
        log.debug(f"Uploader daemon is shutting down")
        executor.shutdown(wait=True)

    def _get_presigned_url_for_put_method(
        self,
        evaluation_id: str,
        remote_object_name: str,
    ) -> str:
        log.check_ne(evaluation_id, "")
        log.check_ne(remote_object_name, "")

        try:
            if not self._api_client:
                log.error("API Client is not initialized")
                raise ValueError("API Client is not initialized")

            response = self._api_client.put(
                f"evaluations/{evaluation_id}/uploading-presigned-url",
                {
                    "processed_uri": remote_object_name,
                },
            )
            response.raise_for_status()
            return response.text.replace('"', "")
        except requests.exceptions.HTTPError as e:
            log.error(f"HTTP error in getting a presigned url: {e}")
            raise HTTPError(
                f"Failed to get presigned URL for {remote_object_name}: {e}",
                status_code=e.response.status_code if e.response else None,
            )

    def _upload_worker(self, index, worker_event) -> None:
        # Individual worker for uploading files. The upload manager creates multiple threads for each of this worker.
        if not (
            self._queue is not None
            and self._worker_event is not None
            and self._daemon_thread is not None
            and self._api_client is not None
            and self._upload_start is not None
            and self._upload_finish is not None
        ):
            raise ValueError("Upload Manager is not initialized")

        log.debug(f"Worker is {index} ready")
        while True:
            if not self._queue.empty():
                item = self._queue.get()
                evaluation_id = item[0]
                remote_object_name = item[1]
                path = item[2]

                log.debug(f"Worker {index} presigned url request")
                presigned_url = self._get_presigned_url_for_put_method(
                    evaluation_id,
                    remote_object_name,
                )
                log.debug(f"Worker {index} presigned url obtained")

                log.debug(f"Worker {index} uploading {path}")
                # Timestamp in ISO 8601.
                upload_start_at = datetime.datetime.now().astimezone().isoformat(timespec="milliseconds")
                self._api_client.put_file_presigned_url(presigned_url, path)
                upload_finish_at = datetime.datetime.now().astimezone().isoformat(timespec="milliseconds")
                log.debug(f"Worker {index} finished uploading {item}")

                self._upload_start[remote_object_name] = upload_start_at
                self._upload_finish[remote_object_name] = upload_finish_at
                self._queue.task_done()
                self._total_uploaded += 1
                log.debug(f"Worker {index} total_uploaded: {self._total_uploaded}")
                if self._pbar:
                    self._pbar.update(1)

            time.sleep(0.1)
            if worker_event.is_set():
                log.debug(f"Worker {index} is done")
                return

    def add_file_to_queue(self, evaluation_id: str, remote_object_name: str, path: str) -> None:
        if not (
            self._queue is not None
            and self._worker_event is not None
            and self._daemon_thread is not None
            and self._api_client is not None
            and self._upload_start is not None
            and self._upload_finish is not None
        ):
            raise ValueError("Upload Manager is not initialized")

        log.debug(f"Added: {path}")
        self._queue.put((evaluation_id, remote_object_name, path))
        self._total_files += 1

    def wait_and_close(self) -> bool:
        if not self._status:
            return False

        if not (self._queue is not None and self._worker_event is not None and self._daemon_thread is not None):
            raise ValueError("Upload Manager is not initialized")
        log.debug(f"total_files: {self._total_files}")
        self._pbar = tqdm(total=self._total_files, dynamic_ncols=True)
        self._pbar.update(self._total_uploaded)

        # Block until all tasks are done.
        log.debug("Queue join")
        self._queue.join()

        # Signal all the workers to exit.
        log.debug("Set exit event to workers")
        self._worker_event.set()
        # Shutdown the executor.
        log.debug("Shutdown uploader daemon")
        self._daemon_thread.join()

        self._pbar.close()
        log.info("All upload work complete.")
        self._status = False
        return True

    def _check_if_initialize(self) -> bool:
        return (
            self._queue is not None
            and self._worker_event is not None
            and self._daemon_thread is not None
            and self._api_client is not None
            and self._upload_start is not None
            and self._upload_finish is not None
        )



from datetime import datetime, timedelta
from typing import Any, Dict, Optional

from podonos.core.base import *
from podonos.common.constant import PODONOS_CONTACT_EMAIL
from podonos.common.enum import EvalType, Language


class EvalConfigDefault:
    TYPE = EvalType.NMOS
    LAN = Language.ENGLISH_AMERICAN
    NUM_EVAL = 10
    DUE_HOURS = 12
    USE_ANNOTATION = False
    AUTO_START = False
    GRANULARITY = 1.0
    MAX_UPLOAD_WORKERS = 20


class EvalConfig:
    _eval_id: str
    _eval_name: str
    _eval_expected_due: str  # Due string in ISO 8601.
    _eval_creation_timestamp: str  # Create a mission timestamp string. Use this as a prefix of uploaded filenames.
    _eval_description: Optional[str] = None
    _eval_type: EvalType = EvalConfigDefault.TYPE
    _eval_language: Language = EvalConfigDefault.LAN
    _eval_granularity: float = EvalConfigDefault.GRANULARITY
    _eval_num: int = EvalConfigDefault.NUM_EVAL
    _eval_expected_due_tzname: Optional[str] = None
    _eval_use_annotation: bool = False
    _eval_auto_start: bool = False
    _max_upload_workers: int = EvalConfigDefault.MAX_UPLOAD_WORKERS

    def __init__(
        self,
        name: Optional[str] = None,
        desc: Optional[str] = None,
        type: str = EvalConfigDefault.TYPE.value,
        lan: str = EvalConfigDefault.LAN.value,
        granularity: float = EvalConfigDefault.GRANULARITY,
        num_eval: int = EvalConfigDefault.NUM_EVAL,
        due_hours: int = EvalConfigDefault.DUE_HOURS,  # TODO: allow floating point hours, e.g. 0.5.
        use_annotation: bool = EvalConfigDefault.USE_ANNOTATION,
        auto_start: bool = EvalConfigDefault.AUTO_START,
        max_upload_workers: int = EvalConfigDefault.MAX_UPLOAD_WORKERS,
    ) -> None:
        self._eval_name = self._valudate_eval_name(name)
        self._eval_description = desc
        self._eval_type = self._validate_eval_type(type)
        self._eval_language = self._validate_eval_language(lan)
        self._eval_num = self._validate_eval_num(num_eval)
        self._eval_granularity = self._validate_eval_granularity(granularity)
        self._eval_expected_due = self._validate_eval_expected_due(due_hours)
        self._eval_expected_due_tzname = self._validate_eval_expected_due_tzname()
        self._eval_creation_timestamp = self._validate_eval_creation_timestamp()
        self._eval_id = self._eval_creation_timestamp
        self._eval_use_annotation = self._validate_eval_use_annotation(use_annotation, type)
        self._eval_auto_start = auto_start
        self._max_upload_workers = max_upload_workers
        self.log_eval_config()

    def log_eval_config(self) -> None:
        log.debug(f"Name: {self._eval_name}")
        log.debug(f"Desc: {self._eval_description}")
        log.debug(f"Eval type: {self._eval_type}")
        log.debug(f"Language: {self._eval_language}")
        log.debug(f"num_eval: {self._eval_num}")
        log.debug(f"Expected due: {self._eval_expected_due} {self._eval_expected_due_tzname}")
        log.debug(f"Evaluation ID: {self._eval_id}")
        log.debug(f"Evaluation use annotation: {self._eval_use_annotation}")
        log.debug(f"Evaluation auto start: {self._eval_auto_start}")
        log.debug(f"Max upload workers: {self._max_upload_workers}")

    @property
    def eval_id(self) -> str:
        return self._eval_id

    @property
    def eval_language(self) -> Language:
        return self._eval_language

    @property
    def eval_type(self) -> EvalType:
        return self._eval_type

    @property
    def eval_creation_timestamp(self) -> str:
        return self._eval_creation_timestamp

    @property
    def eval_use_annotation(self) -> bool:
        return self._eval_use_annotation

    @property
    def eval_auto_start(self) -> bool:
        return self._eval_auto_start

    @property
    def max_upload_workers(self) -> int:
        return self._max_upload_workers

    @eval_id.setter
    def eval_id(self, eval_id: str) -> None:
        self._eval_id = eval_id

    def _valudate_eval_name(self, eval_name: Optional[str]) -> str:
        if not eval_name:
            current = datetime.now()
            return f"{current.year}{current.month}{current.day}{current.hour}{current.minute}{current.second}"
        elif len(eval_name) > 1:
            return eval_name
        else:
            raise ValueError('"name" must be longer than 1.')

    def _validate_eval_type(self, eval_type: str) -> EvalType:
        if eval_type not in [
            EvalType.NMOS.value,
            EvalType.QMOS.value,
            EvalType.SMOS.value,
            EvalType.P808.value,
            EvalType.PREF.value,
        ]:
            raise ValueError(
                f'"type" must be one of {{NMOS, QMOS, SMOS, P808}}. \n'
                + f"Do you want other evaluation types? Let us know at {PODONOS_CONTACT_EMAIL}"
            )
        return EvalType(eval_type)

    def _validate_eval_language(self, eval_language: str) -> Language:
        if eval_language not in [
            Language.ENGLISH_AMERICAN.value,
            Language.ENGLISH_AUSTRALIAN.value,
            Language.ENGLISH_BRITISH.value,
            Language.ENGLISH_CANADIAN.value,
            Language.KOREAN.value,
            Language.MANDARIN.value,
            Language.SPANISH_SPAIN.value,
            Language.SPANISH_MEXICO.value,
            Language.FRENCH.value,
            Language.GERMAN.value,
            Language.JAPANESE.value,
            Language.ITALIAN.value,
            Language.POLISH.value,
            Language.AUDIO.value,
        ]:
            raise ValueError(
                f'"lan" must be one of the supported language strings. '
                + f"See https://www.podonos.com/docs/reference#create-evaluator \n"
                + f"Do you want us to support other languages? Let us know at {PODONOS_CONTACT_EMAIL}."
            )
        return Language(eval_language)

    def _validate_eval_num(self, num_eval: int) -> int:
        if num_eval < 1:
            raise ValueError(f'"num_eval" must be >= 1.')
        return num_eval

    def _validate_eval_granularity(self, granularity: float) -> float:
        if granularity not in [0.5, 1.0]:
            raise ValueError(f'"granularity" must be one of 0.5 and 1.9')
        return granularity

    # TODO: allow floating point hours, e.g. 0.5.
    def _validate_eval_expected_due(self, due_hours: int) -> str:
        if due_hours < 12:
            raise ValueError('"due_hours" must be >=12.')

        due = datetime.now() + timedelta(hours=due_hours)
        return due.astimezone().isoformat(timespec="milliseconds")

    def _validate_eval_expected_due_tzname(self) -> Optional[str]:
        return datetime.now().astimezone().tzname()

    def _validate_eval_creation_timestamp(self) -> str:
        return datetime.now().isoformat(timespec="milliseconds")

    def _validate_eval_use_annotation(self, eval_use_annotation: bool, eval_type: str) -> bool:
        if eval_use_annotation and eval_type not in [
            EvalType.NMOS.value,
            EvalType.QMOS.value,
            EvalType.P808.value,
        ]:
            raise ValueError(f'"eval_type" must be one of {{NMOS, QMOS, P808}} when using "use_annotation"')
        return eval_use_annotation

    def to_dict(self) -> Dict[str, Any]:
        return {
            "eval_id": self._eval_id,
            "eval_name": self._eval_name,
            "eval_description": self._eval_description,
            "eval_type": self._eval_type.value,
            "eval_language": self._eval_language.value,
            "eval_num": self._eval_num,
            "eval_expected_due": self._eval_expected_due,
            "eval_creation_timestamp": self._eval_creation_timestamp,
            "eval_use_annotation": self._eval_use_annotation,
            "eval_auto_start": self._eval_auto_start,
            "max_upload_workers": self._max_upload_workers,
        }

    def to_create_request_dto(self) -> Dict[str, Any]:
        return {
            "title": self._eval_name,
            "internal_name": self._eval_name,
            "description": self._eval_description,
            "language": self._eval_language.value,
            "num_required_etors": self._eval_num,
            "granularity": self._eval_granularity,
            "evaluation_type": self._eval_type.get_type(),
            "use_annotation": self._eval_use_annotation,
            "auto_start": self._eval_auto_start,
        }



class HTTPError(Exception):
    def __init__(self, message, status_code=None, response=None):
        super().__init__(message)
        self.status_code = status_code
        self.response = response



from typing import List, Optional

from podonos.core.base import *


class File:
    _path: str
    _tags: List[str]
    _script: Optional[str]

    def __init__(self, path: str, model_tag: str, tags: List[str] = [], script: Optional[str] = None,
                 is_ref: bool = False) -> None:
        """
        Args:
            path: Path to the file to evaluate. Required.
            model_tag: String that represents the model or group. Required.
            tags: A list of string for file. Optional.
            script: Script of the input audio in text. Optional.
            is_ref: True if this file is to be a reference for an evaluation type that requires a reference.
                    Optiona. Default is False.
        """
        log.check_ne(path, "")
        log.check_ne(model_tag, "")
        self._path = path
        self._model_tag = model_tag
        self._tags = tags
        self._script = script
        self._is_ref = is_ref

    @property
    def path(self) -> str:
        return self._path

    @property
    def model_tag(self) -> str:
        return self._model_tag

    @property
    def tags(self) -> List[str]:
        return self._tags

    @property
    def script(self) -> Optional[str]:
        return self._script

    @property
    def is_ref(self) -> Optional[bool]:
        return self._is_ref



import os
import requests
from abc import ABC, abstractmethod
from typing import Tuple, Dict, List, Optional

from podonos.core.base import *
from podonos.common.constant import *
from podonos.common.enum import EvalType, QuestionFileType
from podonos.common.exception import HTTPError
from podonos.common.util import generate_random_name
from podonos.core.api import APIClient
from podonos.core.audio import Audio
from podonos.core.config import EvalConfig
from podonos.core.evaluation import Evaluation
from podonos.core.file import File
from podonos.core.query import Query
from podonos.core.upload_manager import UploadManager


class Evaluator(ABC):
    """Evaluator for a single type of evaluation session."""

    _initialized: bool = False
    _api_client: APIClient
    _api_key: Optional[str] = None
    _eval_config: Optional[EvalConfig] = None
    _supported_evaluation_types: List[EvalType]
    _evaluation: Optional[Evaluation] = None

    # Upload manager. Lazy initialization when used for saving resources.
    _upload_manager: Optional[UploadManager] = None

    # Custom Query.
    _query: Optional[Query] = None

    # Contains the metadata for all the audio files for evaluation.
    _eval_audios: List[List[Audio]] = []
    _eval_audio_json = []

    def __init__(self, api_client: APIClient, eval_config: Optional[EvalConfig] = None):
        log.check(api_client, "api_client is not initialized.")
        self._api_client = api_client
        self._api_key = api_client.api_key
        self._eval_config = eval_config
        self._initialized = True
        self._eval_audios = []
        self._eval_audio_json = []
        self._evaluation = self._create_evaluation()

    def _init_eval_variables(self):
        """Initializes the variables for one evaluation session."""
        self._initialized = False
        self._api_key = None
        self._eval_config = None
        self._eval_audios = []
        self._eval_audio_json = []

    @abstractmethod
    def add_file(self, file: File) -> None:
        pass

    @abstractmethod
    def add_files(self, file0: File, file1: File) -> None:
        pass

    def get_evaluation_id(self) -> str:
        """
        Returns the evaluation id for this evaluator

        Returns:
            Evaluation id in string
        """
        assert self._evaluation
        return self._evaluation.id

    def close(self) -> Dict[str, str]:
        """Closes the file uploading and evaluation session.
        This function holds until the file uploading finishes.

        Returns:
            JSON object containing the uploading status.

        Raises:
            ValueError: if this function is called before calling init().
        """
        log.debug("Closing the evaluator")
        if not self._initialized or self._eval_config is None:
            raise ValueError("No evaluation session is open.")

        if self._eval_config.eval_type not in self._supported_evaluation_types:
            raise ValueError("Not supported evaluation type")

        if self._upload_manager is None:
            raise ValueError("Upload Manager is not defined")

        # Wait until file uploading finishes.
        log.debug("Wait until the upload manager shuts down all the upload workers")
        assert self._upload_manager.wait_and_close()

        log.info("Uploading the final pieces...")

        # Create a template if custom query exists
        self._create_template_with_question_and_evaluation()

        # Insert File data into database
        audios = [audio for audio_list in self._eval_audios for audio in audio_list]
        for i in range(0, len(audios), 500):
            self._create_files_of_evaluation(audios[i : i + 500])

        # Get the upload time & finish time.
        upload_start, upload_finish = self._upload_manager.get_upload_time()
        for audio_list in self._eval_audios:
            audio_json_list = []
            for audio in audio_list:
                remote_object_name = audio.remote_object_name
                upload_start_at = upload_start[remote_object_name]
                upload_finish_at = upload_finish[remote_object_name]
                audio.set_upload_at(upload_start_at, upload_finish_at)
                audio_json_list.append(audio.to_dict())
            self._eval_audio_json.append(audio_json_list)

        # Create a json.
        session_json = self._eval_config.to_dict()
        session_json["query"] = self._query.to_dict() if self._query else None
        session_json["files"] = self._eval_audio_json

        presigned_url = self._get_presigned_url_for_put_method(
            self.get_evaluation_id(),
            "session.json",
        )

        try:
            response = self._api_client.put_json_presigned_url(url=presigned_url, data=session_json, headers={"Content-type": "application/json"})
            response.raise_for_status()
        except requests.exceptions.HTTPError as e:
            log.error(f"HTTP error in uploading a json: {e}")
            raise HTTPError(
                f"Failed to upload session info json: {e}",
                status_code=e.response.status_code if e.response else None,
            )

        if self._eval_config.eval_auto_start:
            log.info(f"{TerminalColor.OK}Upload finished. The evaluation will start immediately.{TerminalColor.ENDC}")
        else:
            log.info(f"{TerminalColor.OK}Upload finished. Please start the evaluation at {PODONOS_WORKSPACE}." f"{TerminalColor.ENDC}")

        # Initialize variables.
        self._init_eval_variables()
        return {"status": "ok"}

    def _get_eval_config(self) -> EvalConfig:
        if not self._eval_config:
            raise ValueError("Evaluator is not initialized")
        return self._eval_config

    def _create_evaluation(self) -> Evaluation:
        """
        Create a new evaluation based on the evaluation configuration

        Raises:
            HTTPError: If the value is invalid

        Returns:
            Evaluation: Get new evaluation information
        """
        log.debug("Creates evaluation")
        eval_config = self._get_eval_config()
        try:
            response = self._api_client.post("evaluations", data=eval_config.to_create_request_dto())
            response.raise_for_status()
            evaluation = Evaluation.from_dict(response.json())
            log.info(f"Evaluation is generated: {evaluation.id}")
            return evaluation
        except Exception as e:
            raise HTTPError(f"Failed to create the evaluation: {e}")

    def _upload_one_file(
        self,
        evaluation_id: str,
        remote_object_name: str,
        path: str,
    ) -> None:
        """
        Start uploading one file to server.

        Args:
            evaluation_id: New evaluation's id.
            remote_object_name: Path to the remote file name.
            path: Path to the local file.
        Returns:
            None
        """
        log.check_notnone(evaluation_id)
        log.check_notnone(remote_object_name)
        log.check_notnone(path)
        log.check_ne(evaluation_id, "")
        log.check_ne(remote_object_name, "")
        log.check_ne(path, "")

        # Get the presigned URL for one file
        log.debug(f"Adding to queue: {path}")
        if not self._eval_config:
            raise ValueError("No evaluation session is open.")

        # Lazy initialization of upload manager.
        if self._upload_manager is None:
            log.debug(f"max_upload_workers: {self._eval_config.max_upload_workers}")
            self._upload_manager = UploadManager(
                api_client=self._api_client,
                max_workers=self._eval_config.max_upload_workers,
            )

        if self._upload_manager:
            self._upload_manager.add_file_to_queue(evaluation_id, remote_object_name, path)
        return

    def _get_presigned_url_for_put_method(
        self,
        evaluation_id: str,
        remote_object_name: str,
    ) -> str:
        log.check_ne(evaluation_id, "")
        log.check_ne(remote_object_name, "")

        try:
            response = self._api_client.put(
                f"evaluations/{evaluation_id}/uploading-presigned-url",
                {
                    "processed_uri": remote_object_name,
                },
            )
            response.raise_for_status()
            return response.text.replace('"', "")
        except requests.exceptions.HTTPError as e:
            log.error(f"HTTP error in getting a presigned url: {e}")
            raise HTTPError(
                f"Failed to get presigned URL for {remote_object_name}: {e}",
                status_code=e.response.status_code if e.response else None,
            )

    def _create_files_of_evaluation(self, audios: List[Audio]):
        try:
            response = self._api_client.put(
                f"evaluations/{self.get_evaluation_id()}/files",
                {"files": [audio.to_create_file_dict() for audio in audios]},
            )
            response.raise_for_status()
        except requests.exceptions.HTTPError as e:
            log.error(f"HTTP error in adding file meta: {e}")
            raise HTTPError(
                f"Failed to create evaluation files: {e}",
                status_code=e.response.status_code if e.response else None,
            )

    def _create_template_with_question_and_evaluation(
        self,
    ) -> None:
        eval_config = self._get_eval_config()
        try:
            if not self._query:
                return None

            response = self._api_client.post(
                "templates",
                {
                    "evaluation_id": self.get_evaluation_id(),
                    "title": self._query.title,
                    "description": self._query.description,
                    "language": eval_config.eval_language.value,
                },
            )
            response.raise_for_status()
        except Exception as e:
            log.error(f"HTTP error in evaluation template generation: {e}")
            raise e

    def _set_audio(
        self,
        file: File,
        group: Optional[str],
        type: QuestionFileType,
        order_in_group: int,
    ) -> Audio:
        log.check_ne(file.path, "")

        valid_path = self._validate_path(file.path)
        remote_object_name = self._get_remote_object_name()
        original_path, remote_path = self._process_original_path_and_remote_object_path_into_posix_style(valid_path, remote_object_name)

        log.debug(f"remote_object_name: {remote_object_name}")
        return Audio(
            path=valid_path,
            name=original_path,
            remote_object_name=remote_path,
            script=file.script,
            tags=file.tags,
            model_tag=file.model_tag,
            is_ref=file.is_ref if file.is_ref else False,
            group=group,
            type=type,
            order_in_group=order_in_group,
        )

    @staticmethod
    def _validate_path(path: str) -> str:
        if not os.path.isfile(path):
            raise FileNotFoundError(f"File {path} doesn't exist")

        if not os.access(path, os.R_OK):
            raise FileNotFoundError(f"File {path} isn't readable")
        return path

    def _get_remote_object_name(self) -> str:
        eval_config = self._get_eval_config()
        remote_object_name = os.path.join(eval_config.eval_creation_timestamp, generate_random_name())
        return remote_object_name

    @staticmethod
    def _process_original_path_and_remote_object_path_into_posix_style(original_path: str, remote_object_path: str) -> Tuple[str, str]:
        posix_original_path = original_path.replace("\\", "/")
        posix_remote_object_path = remote_object_path.replace("\\", "/")
        return posix_original_path, posix_remote_object_path



import os
import podonos
import requests
import mimetypes
import importlib.metadata

from requests import Response
from typing import Dict, Any, Optional
from packaging.version import Version

from podonos.common.constant import *
from podonos.common.exception import HTTPError
from podonos.core.base import *


class APIVersion:
    _minimum: Version
    _recommended: Version
    _latest: Version

    def __init__(self, minimum: str, recommended: str, latest: str):
        self._minimum = Version(minimum)
        self._recommended = Version(recommended)
        self._latest = Version(latest)

    @property
    def minimum(self) -> Version:
        return self._minimum

    @property
    def recommended(self) -> Version:
        return self._recommended

    @property
    def latest(self) -> Version:
        return self._latest


class APIClient:
    _api_key: str
    _api_url: str
    _headers: Dict[str, str] = {}

    def __init__(self, api_key: str, api_url: str):
        self._api_key = api_key
        self._api_url = api_url
        self._headers = {"X-API-KEY": self._api_key}

    @property
    def api_key(self) -> str:
        return self._api_key

    @property
    def api_url(self) -> str:
        return self._api_url

    def initialize(self) -> bool:
        self._check_minimum_version()

        response = self.get("customers/verify/api-key")
        if response.text != "true":
            raise ValueError(TerminalColor.FAIL + f"Invalid API key: {self._api_key}" + TerminalColor.ENDC)
        return True

    def add_headers(self, key: str, value: str) -> None:
        log.check_notnone(key)
        log.check_ne(key, "")
        log.check_notnone(value)
        self._headers[key] = value

    def get(
        self,
        endpoint: str,
        params: Optional[Dict[str, str]] = None,
        headers: Optional[Dict[str, str]] = None,
    ) -> Response:
        log.check_notnone(endpoint)
        log.check_ne(endpoint, "")
        request_header = self._headers if headers is None else headers
        response = requests.get(f"{self._api_url}/{endpoint}", headers=request_header, params=params)
        return response

    def post(
        self,
        endpoint: str,
        data: Dict[str, Any],
        headers: Optional[Dict[str, str]] = None,
    ) -> Response:
        log.check_notnone(endpoint)
        log.check_ne(endpoint, "")
        request_header = self._headers if headers is None else headers
        response = requests.post(f"{self._api_url}/{endpoint}", headers=request_header, json=data)
        return response

    def put(
        self,
        endpoint: str,
        data: Dict[str, Any],
        headers: Optional[Dict[str, str]] = None,
    ) -> Response:
        log.check_notnone(endpoint)
        log.check_ne(endpoint, "")
        request_header = self._headers if headers is None else headers
        response = requests.put(f"{self._api_url}/{endpoint}", headers=request_header, json=data)
        return response

    def put_file_presigned_url(self, url: str, path: str) -> Response:
        log.check_notnone(url)
        log.check_notnone(path)
        log.check_ne(url, "")
        log.check_ne(path, "")
        log.check(os.path.isfile(path), f"{path} doesn't exist")
        log.check(os.access(path, os.R_OK), f"{path} isn't readable")

        try:
            response = requests.put(
                url,
                data=open(path, "rb"),
                headers={"Content-Type": self._get_content_type_by_filename(path)},
            )
            return response
        except requests.exceptions.RequestException as e:
            log.error(f"HTTP error in uploading a file to presigned URL: {e}")
            raise HTTPError(
                f"Failed to Upload File {path}: {e}",
                status_code=e.response.status_code if e.response else None,
            )

    def put_json_presigned_url(self, url: str, data: Dict[str, Any], headers: Optional[Dict[str, str]] = None) -> Response:
        log.check_notnone(url)
        log.check_ne(url, "")

        log.debug("JSON data")
        for key, value in data.items():
            log.debug(f"{key}: {value}")
        if headers:
            log.debug("Headers")
            for key, value in data.items():
                log.debug(f"{key}: {value}")

        try:
            response = requests.put(url, json=data, headers=headers)
            return response
        except requests.exceptions.RequestException as e:
            log.error(f"HTTP error in uploading a json to presigned url: {e}")
            raise HTTPError(
                f"Failed to Upload JSON {data}: {e}",
                status_code=e.response.status_code if e.response else None,
            )

    @staticmethod
    def _get_content_type_by_filename(path: str) -> str:
        log.check_notnone(path)
        log.check_ne(path, "")
        _, ext = os.path.splitext(path)
        if ext == ".wav":
            return "audio/wav"
        elif ext == ".mp3":
            return "audio/mpeg"
        elif ext == ".json":
            return "application/json"
        return "application/octet-stream"

    def _check_minimum_version(self) -> bool:
        response = self.get("version/sdk")
        api_version = APIVersion(**response.json())

        current_version = self._get_podonos_version()
        log.debug(f"current package version: {current_version}")

        if Version(current_version) >= api_version.recommended:
            return True

        if Version(current_version) >= api_version.minimum:
            print(
                "The current podonos package version is {current_version} "
                "while a newer version {api_version.latest} is available\n"
                "Please upgrade by 'pip install podonos --upgrade'"
            )
            return True

        # This version is lower than the minimum required version. Cannot proceed.
        print(
            TerminalColor.FAIL + f"The current podonos package version is {current_version} "
            f"while the minimum supported version is {api_version.minimum}"
            + TerminalColor.ENDC
            + "\n"
            + TerminalColor.BOLD
            + "Please upgrade"
            + TerminalColor.ENDC
            + f" by 'pip install podonos --upgrade'"
        )
        raise ValueError(f"Minimum supported version is {api_version.minimum}")

    @staticmethod
    def _get_podonos_version():
        try:
            # Try to get the version using importlib.metadata
            return importlib.metadata.version("podonos")
        except importlib.metadata.PackageNotFoundError:
            # Fallback to __version__ from podonos package if importlib fails
            return podonos.__version__
